@@ -31,14 +31,17 @@
 
 #include <string.h>
 #include <glib/gi18n-lib.h>
+#include <gdk-pixbuf/gdk-pixbuf.h>
 #include <gtk/gtkalignment.h> 
 #include <gtk/gtkbox.h> 
 #include <gtk/gtkbutton.h>
 #include <gtk/gtkcellrenderertext.h>
 #include <gtk/gtkcellrenderertoggle.h>
+#include <gtk/gtkcellrendererpixbuf.h>
 #include <gtk/gtkentry.h>
 #include <gtk/gtkhbbox.h>
 #include <gtk/gtkimage.h>
+#include <gtk/gtkicontheme.h>
 #include <gtk/gtklabel.h>
 #include <gtk/gtkliststore.h>
 #include <gtk/gtkscrolledwindow.h>
@@ -68,6 +71,7 @@ struct _EelMimeApplicationChooserDetails {
 
 enum {
 	COLUMN_DEFAULT,
+	COLUMN_ICON,
 	COLUMN_NAME,
 	COLUMN_ID,
 	NUM_COLUMNS
@@ -208,6 +212,7 @@ create_tree_view (EelMimeApplicationChooser *chooser)
 	
 	store = gtk_list_store_new (NUM_COLUMNS,
 				    G_TYPE_BOOLEAN,
+				    GDK_TYPE_PIXBUF,
 				    G_TYPE_STRING,
 				    G_TYPE_STRING);
 	gtk_tree_view_set_model (GTK_TREE_VIEW (treeview),
@@ -228,6 +233,14 @@ create_tree_view (EelMimeApplicationChooser *chooser)
 							   NULL);
 	gtk_tree_view_append_column (GTK_TREE_VIEW (treeview), column);
 
+	renderer = gtk_cell_renderer_pixbuf_new ();
+	column = gtk_tree_view_column_new_with_attributes (_("Icon"),
+							   renderer,
+							   "pixbuf",
+							   COLUMN_ICON,
+							   NULL);
+	gtk_tree_view_append_column (GTK_TREE_VIEW (treeview), column);
+
 	chooser->details->toggle_renderer = renderer;
 	renderer = gtk_cell_renderer_text_new ();
 	column = gtk_tree_view_column_new_with_attributes (_("Name"),
@@ -391,6 +404,10 @@ refresh_model (EelMimeApplicationChooser *chooser)
 		gboolean is_default;
 		GnomeVFSMimeApplication *application;
 		char *escaped;
+		const char *icon;
+		GdkPixbuf *pixbuf;
+
+		pixbuf = NULL;
 
 		application = l->data;
 		
@@ -399,14 +416,44 @@ refresh_model (EelMimeApplicationChooser *chooser)
 
 		escaped = g_markup_escape_text (application->name, -1);
 
+		icon = gnome_vfs_mime_application_get_icon (application);
+
+		if (icon != NULL) {
+			if (g_path_is_absolute (icon)) {
+				pixbuf = gdk_pixbuf_new_from_file_at_size (icon, 24, 24, NULL);
+			}
+			if (pixbuf == NULL) {
+				pixbuf = gtk_icon_theme_load_icon (gtk_icon_theme_get_default (),
+								   icon, 24, 0, NULL);
+			}
+			if (pixbuf == NULL && strrchr (icon, '.') != NULL) {
+				char *name;
+				name = g_strndup (icon, strlen (icon) - strlen (strrchr (icon, '.')));
+				pixbuf = gtk_icon_theme_load_icon (gtk_icon_theme_get_default (),
+								   name, 24, 0, NULL);
+				g_free (name);
+			}
+	
+			if (pixbuf == NULL) {
+				char *filename;
+				filename = g_strdup_printf (DATADIR"/pixmaps/%s", icon);
+				pixbuf = gdk_pixbuf_new_from_file_at_size (filename, 24, 24, NULL);
+				g_free (filename);
+			}
+		}
+
 		gtk_list_store_append (chooser->details->model, &iter);
 		gtk_list_store_set (chooser->details->model, &iter,
 				    COLUMN_DEFAULT, is_default,
+				    COLUMN_ICON, pixbuf,
 				    COLUMN_NAME, escaped,
 				    COLUMN_ID,
 				    gnome_vfs_mime_application_get_desktop_id (application), 
 				    -1);
+
 		g_free (escaped);
+		if (pixbuf != NULL)
+			g_object_unref(pixbuf);
 	}
 
 	selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (chooser->details->treeview));
